; tachometer, speedometer for motobike
;*****************
.include "tn2313def.inc"
;*****************
; pcb version
.equ PCB	= 1
; constants
; in-out port constants
.equ E		= PORTA0
.equ RS		= PORTA1
.equ RSTPIN	= PORTA2
;
.equ OIL		= PORTD0
.equ NEUTRAL	= PORTD1
.equ LIGHT		= PORTD4
.equ POWER_CONTROL	= PORTD5
.equ RESETPATH	= PORTD6
; time constants
.equ QUARTZ		= 4000; quartz frequency, kHz
.equ PRESCALER	= 256
.equ PRESCALER0	= 256
.equ REFRESH_TIME	= 50;125; ms
.equ REFRESH_CNT	= (QUARTZ * REFRESH_TIME / PRESCALER)
;
.equ PPP	= 188; sm, path per pulse
.equ SPEED_CNSTNT	= (PPP * 36 * QUARTZ / PRESCALER)
; LCD constants
.equ LCD_NOTDOT_CODE	= '-'; symbol not dot code
.equ LCD_DOT_CODE	= 11;0xff; symbol dot code
.equ LCD_NOTDATA_CODE	= '.'; code for not imput data
.equ LCD_OIL_CODE	= 8;'!'
.equ LCD_NEUTRAL_CODE	= 9;'N'
.equ LCD_LIGHT_CODE	= 10;0xfd
.equ DOTS_SHIFT	= 1; 0 - from 0pps, 1 - treshold 1/13 ppsmax
.equ PPS_MAX	= 12000
.equ PPSDOTS	= 12; must be <= LCD_STR_LENGTH
; memory allocate
.equ LCDbuff	= SRAM_START
.equ LCD_STR_LENGTH	= 16; must be >= PPSDOTS
.equ LCDbuffSTR1	= LCDbuff
.equ LCDbuffSTR2	= LCDbuff + LCD_STR_LENGTH
.equ LCD_SHIFT_SPEED	= 0; 0...LCD_STR_LENGTH-3
.equ LCD_SHIFT_PATH = 8; 0...LCD_STR_LENGTH-8
; flash memory allocate
.equ FMPATH_1	= 0
.equ FMPATH_2	= FMPATH_1 + 1
.equ FMPATH_3	= FMPATH_2 + 1
.equ FMPATH_4	= FMPATH_3 + 1
.equ FMPATH_5	= FMPATH_4 + 1
; variables
.def old_pwrcntrl	= r0
.def old_resetbtn	= r1
.def dTL	= r2
.def dTH	= r3
;.def old_pin	= r4
.def timerL	= r5
.def timerH	= r6
.def path_1		= r7
.def path_2		= r8
.def path_3		= r9
.def path_4		= r10
.def path_5		= r11
.def st_store1	= r12
.def st_store2	= r13
.def eeprom_data	= r14
.def zero		= r15

.def temp	= r16
.def tmp1	= r17
.def tmp2	= r18
.def tmp3	= r19
.def tmp4	= r20
.def tmp5	= r21
.def CtimerL	= r22
.def CtimerH	= r23
.def speed_cntrl	= r24
.def itemp	= r25
; xh.xl tachometer	= r26, r27
; yh.yl		= r28, r29
; zh.zl lcd			= r30, r31
;-----------------
; check constants
.IF (LCD_STR_LENGTH - PPSDOTS) < 0
.ERROR "dots more than lcd string!"
.ENDIF
.IF DOTS_SHIFT < 0 || DOTS_SHIFT > 1
.ERROR "dots shift must be 0 or 1!"
.ENDIF
.IF (REFRESH_TIME < 50) || (1000 / REFRESH_TIME * REFRESH_TIME != 1000)
.ERROR "refresh time not correct!"
.ENDIF
;*****************
.eseg
;*****************
.cseg
.org 0
	rjmp RESET ; Reset Handler
	rjmp TACHOMETER;INT0 ; External Interrupt0 Handler
	rjmp SPEEDOMETER;INT1 ; External Interrupt1 Handler
	reti;rjmp TIM1_CAPT ; Timer1 Capture Handler
	rjmp SCREEN_REFRESH;TIM1_COMPA ; Timer1 CompareA Handler
	reti ;rjmp TIM1_OVF ; Timer1 Overflow Handler
	rjmp POWERDOWN_CONTROL;rjmp TIM0_OVF ; Timer0 Overflow Handler
	reti;rjmp USART0_RXC ; USART0 RX Complete Handler
	reti;rjmp USART0_DRE ; USART0,UDR Empty Handler
	reti;rjmp USART0_TXC ; USART0 TX Complete Handler
	reti;rjmp ANA_COMP ; Analog Comparator Handler
	reti;rjmp PATH_STORE;rjmp PCINT ; Pin Change Interrupt
	rjmp MIN_SPEED_CNTRL;rjmp TIMER1_COMPB ; Timer1 Compare B Handler
	reti;rjmp TIMER0_COMPA ; Timer0 Compare A Handler
	reti;rjmp TIMER0_COMPB ; Timer0 Compare B Handler
	reti;rjmp USI_START ; USI Start Handler
	reti;rjmp USI_OVERFLOW ; USI Overflow Handler
	reti;rjmp EE_READY ; EEPROM Ready Handler
	reti;rjmp WDT_OVERFLOW ; Watchdog Overflow Handler
;****************
; program info
.db "motobike tachometer"
.db "(c) 2006 V*D*V v1.0"
;****************
; initialisation
RESET:
	cli
	ldi	temp, low(RAMEND)
	out	spl, temp
;----------------
	wdr
; set modes
;	ldi	temp, ((0 << WDE) | (0 << WDCE) | (0 << WDP3) | (1 << WDP2) | (1 << WDP1) | (0 << WDP0))
;	out	WDTCR, temp
; watch dog timer initialization
	ldi	temp, (1 << WDCE) | (1 << WDE)
	out	WDTCSR, temp
	ldi	temp, (1 << WDE) | (0 << WDCE) | (0 << WDIE) | (0 << WDP3) | (1 << WDP2) | (1 << WDP1) | (0 << WDP0)
	out	WDTCSR, temp
;----------------
	sbi	ACSR, ACD; comparator disable
	clr	zero
	out	MCUSR, zero
	out	CLKPR, zero
; sleep mode=idle
	ldi	temp, (0 << PUD)|(0 << SE)|(0 << SM1)|(0 << SM0)|(1 << ISC11)|(1 << ISC10)|(1 << ISC01)|(1 << ISC00)
	out	MCUCR, temp
;----------------
; set default port states, PUD in MCUSR must be 0
	ldi	temp, (0 << E) | (0 << RS) | (1 << RSTPIN)
	out PORTA, temp
	out	PORTB, zero
	ldi	temp, (1 << RESETPATH)
	out	PORTD, temp
; port directions
; DDR 1 - output, PORT must be 0
; DDR 0 - input, if PORT=0 - tristate
	ldi	temp, (1 << E) | (1 << RS)
	out	DDRA, temp
	ser	temp
	out	DDRB, temp
	out	DDRD, zero
;---------------
	rcall	LCDinit
	rcall	WriteCharacter
;---------------
; init data
; tachometer
	clr	xl
	clr	xh
; path read
	ldi	temp, FMPATH_1
	rcall	EEPROM_read
	mov	path_1, temp
	ldi	temp, FMPATH_2
	rcall	EEPROM_read
	mov	path_2, temp
	ldi	temp, FMPATH_3
	rcall	EEPROM_read
	mov	path_3, temp
	ldi	temp, FMPATH_4
	rcall	EEPROM_read
	mov	path_4, temp
	ldi	temp, FMPATH_5
	rcall	EEPROM_read
	mov	path_5, temp
;----------------
; set timer settings for power control
.IF PRESCALER0 == 1024
	ldi	temp, (1<<CS02)|(0<<CS01)|(1<<CS00); timer prescaler clk/1024
.ELSE
.IF PRESCALER0 == 256
	ldi	temp, (1<<CS02)|(0<<CS01)|(0<<CS00); timer prescaler clk/256
.ELSE
.IF PRESCALER0 == 64
	ldi	temp, (0<<CS02)|(1<<CS01)|(1<<CS00); timer prescaler clk/64
.ELSE
.IF PRESCALER0 == 8
	ldi	temp, (0<<CS02)|(1<<CS01)|(0<<CS00); timer prescaler clk/8
.ELSE
.IF PRESCALER0 == 1
	ldi	temp, (0<<CS02)|(0<<CS01)|(1<<CS00); timer prescaler clk/1
.ELSE
.IF PRESCALER0 == 0
	ldi	temp, (0<<CS02)|(0<<CS01)|(0<<CS00); timer prescaler no clk
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
	out	TCCR0B, temp
;----------------
; set timer settings for measuring CTC(1<<WGM12) normal(0<<WGM12)
.IF PRESCALER == 1024
	ldi	temp, (0<<WGM13)|(0<<WGM12)|(1<<CS12)|(0<<CS11)|(1<<CS10); timer prescaler clk/1024
.ELSE
.IF PRESCALER == 256
	ldi	temp, (0<<WGM13)|(0<<WGM12)|(1<<CS12)|(0<<CS11)|(0<<CS10); timer prescaler clk/256
.ELSE
.IF PRESCALER == 64
	ldi	temp, (0<<WGM13)|(0<<WGM12)|(0<<CS12)|(1<<CS11)|(1<<CS10); timer prescaler clk/64
.ELSE
.IF PRESCALER == 8
	ldi	temp, (0<<WGM13)|(0<<WGM12)|(0<<CS12)|(1<<CS11)|(0<<CS10); timer prescaler clk/8
.ELSE
.IF PRESCALER == 1
	ldi	temp, (0<<WGM13)|(0<<WGM12)|(0<<CS12)|(0<<CS11)|(1<<CS10); timer prescaler clk/1
.ELSE
.IF PRESCALER == 0
	ldi	temp, (0<<WGM13)|(0<<WGM12)|(0<<CS12)|(0<<CS11)|(0<<CS10); timer prescaler no clk
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
.ENDIF
	out	TCCR1B, temp
;-----------------
; timer init
	out	TCCR1A, zero
	out	TCNT1H, zero
	out	TCNT1L, zero
	ldi	temp, high(REFRESH_CNT)
	out	OCR1AH, temp
	ldi	temp, low(REFRESH_CNT)
	out	OCR1AL, temp
;	ser	temp
; speedometer
	out	OCR1BH, zero
	out	OCR1BL, zero
	clr	timerL
	clr	timerH
	ldi	speed_cntrl, 1
; timer interrupt configure
; on CTC mode, set data
;	ldi	temp, (1 << OCIE1A)
;	out	TIMSK, temp
; compare match A and B interrupt enable, overflow interrupt enable
	ldi	temp, (1 << OCIE1A) | (1 << OCIE1B) | (1 << TOIE0);(1 << TOIE1)
	out	TIMSK, temp
;---------------
; external interrupt configure for pulse and power control
	out	PCMSK, zero
	ldi	temp, (1 << INT1) | (1 << INT0) | (0 << PCIE)
	out	GIMSK, temp
	ser	temp
	mov	old_pwrcntrl, temp
	mov	old_resetbtn, zero
	out	EIFR, temp
; main
	sei
MAIN:
	wdr
	ldi	temp, (0 << PUD)|(1 << SE)|(0 << SM1)|(0 << SM0)|(1 << ISC11)|(1 << ISC10)|(1 << ISC01)|(1 << ISC00)
	out	MCUCR, temp
	sleep
	rjmp	MAIN
;****************
; tachometer pulse (External Interrupt0 Handler)
TACHOMETER:
	in	st_store1, SREG
	adiw	XL, 1
	out	SREG, st_store1
	reti
;****************
; speedometer pulse (External Interrupt1 Handler)
SPEEDOMETER:
	in	st_store1, SREG
; time calculate
	cpi	speed_cntrl, 255
	adc	speed_cntrl, zero
	in	CtimerL, TCNT1L
	in	CtimerH, TCNT1H
	mov	dTL, CtimerL
	mov	dTH, CtimerH
	subi	CtimerL, 1
	sbc		CtimerH, zero
	out	OCR1BH, CtimerH
	out	OCR1BL, CtimerL
	sub	dTL, timerL
	sbc	dTH, timerH
	in	timerL, TCNT1L
	in	timerH, TCNT1H
; path calculate
	ldi	itemp, PPP
	add	path_1, itemp
	adc	path_2, zero
	adc	path_3, zero
	adc	path_4, zero
	adc	path_5, zero
	out	SREG, st_store1
	reti
;****************
;(Timer1 CompareB Handler)
MIN_SPEED_CNTRL:
;	in	st_store1, SREG
	ldi	speed_cntrl, 1
;	out	SREG, st_store1
	reti
;****************
POWERDOWN_CONTROL:
	in	st_store1, SREG
	mov	itemp, temp
	mov	temp, old_pwrcntrl
	in	old_pwrcntrl, PIND
	sbrc	old_pwrcntrl, POWER_CONTROL
	rjmp	NoPathStore
	eor	temp, old_pwrcntrl
	andi	temp, (1 << POWER_CONTROL)
	breq	NoPathStore
	rcall	PathStore
NoPathStore:
	mov	temp, itemp
	out	SREG, st_store1
	reti
;****************
PathStore:
	ldi	temp, FMPATH_1
	mov	eeprom_data, path_1
	rcall	EEPROM_write
	ldi	temp, FMPATH_2
	mov	eeprom_data, path_2
	rcall	EEPROM_write
	ldi	temp, FMPATH_3
	mov	eeprom_data, path_3
	rcall	EEPROM_write
	ldi	temp, FMPATH_4
	mov	eeprom_data, path_4
	rcall	EEPROM_write
	ldi	temp, FMPATH_5
	mov	eeprom_data, path_5
	rcall	EEPROM_write
	ret
;****************
ClearPath:
	ldi	temp, FMPATH_1
	mov	eeprom_data, zero
	rcall	EEPROM_write
	ldi	temp, FMPATH_2
	mov	eeprom_data, zero
	rcall	EEPROM_write
	ldi	temp, FMPATH_3
	mov	eeprom_data, zero
	rcall	EEPROM_write
	ldi	temp, FMPATH_4
	mov	eeprom_data, zero
	rcall	EEPROM_write
	ldi	temp, FMPATH_5
	mov	eeprom_data, zero
	rcall	EEPROM_write
	cli
	clr	path_1
	clr	path_2
	clr	path_3
	clr	path_4
	clr	path_5
	sei
	ret
;****************
; screen refresh (Timer1 CompareA Handler)
SCREEN_REFRESH:
	in	st_store2, SREG
	in	tmp1, TCNT1L
	in	tmp2, TCNT1H
	ldi	temp, low(REFRESH_CNT)
	add	tmp1, temp
	ldi	temp, high(REFRESH_CNT)
	adc	tmp2, temp
	out	OCR1AH, tmp2
	out	OCR1AL, tmp1
; tahometer (pps) calculate
; pps = 1000 / refresh time * pulse quantity
; refresh time = 1000*PRESCALER*REFRESH_TIME/QUARTZ (ms)
; pps = pulse quantity*QUARTZ*1000/PRESCALER/REFRESH_TIME
;
; dots line calculate
; trsh = ppsmax*Ndot/dotsquantity
	ldi	tmp1, low(REFRESH_TIME * (PPS_MAX - PPS_MAX / (PPSDOTS + DOTS_SHIFT)) / 1000)
	ldi	tmp2, high(REFRESH_TIME * (PPS_MAX - PPS_MAX / (PPSDOTS + DOTS_SHIFT)) / 1000)
	ldi	ZL, low(LCDbuffSTR2 + PPSDOTS)
	ldi	ZH, high(LCDbuffSTR2 + PPSDOTS)
	movw	YL, XL
	sei
;
	clr	XL
	clr	XH
	ldi	temp, PPSDOTS
DotsLoop:
	ldi	tmp3, LCD_NOTDOT_CODE
	cp	YL, tmp1
	cpc	YH, tmp2
	brcs	DotsNotPoint
	ldi	tmp3, LCD_DOT_CODE
DotsNotPoint:
	st	-Z, tmp3
	subi	tmp1, low(PPS_MAX * REFRESH_TIME / (PPSDOTS + DOTS_SHIFT) / 1000)
	sbci	tmp2, high(PPS_MAX * REFRESH_TIME / (PPSDOTS + DOTS_SHIFT) / 1000)
	dec	temp
	brne	DotsLoop
; speed calculate
; speed = PPP(sm) * 36 * QUARTZ / PRESCALER / dt
; speed = SPEED_CNSTNT / dt
	clr	tmp1; speed = 0
	cpi	speed_cntrl, 3
	brcs	NoSpeed
	cli
	mov	tmp4, dTL
	mov	tmp5, dTH
	sei
	rcall	TimeToSpeed
NoSpeed:
	rcall	SpeedToView
; path calculate
	rcall	PathToView
; lamp
	ldi	ZL, low(LCDbuffSTR2 + PPSDOTS + 1)
	ldi	ZH, high(LCDbuffSTR2 + PPSDOTS + 1)
;
	ldi	temp, LCD_NOTDATA_CODE
	sbic	PIND, OIL
	ldi	temp, LCD_OIL_CODE
	st	Z+, temp
;
	ldi	temp, LCD_NOTDATA_CODE
	sbic	PIND, NEUTRAL
	ldi	temp, LCD_NEUTRAL_CODE
	st	Z+, temp
;
	ldi	temp, LCD_NOTDATA_CODE
	sbic	PIND, LIGHT
	ldi	temp, LCD_LIGHT_CODE
	st	Z+, temp
;
	rcall	MemToLCD
; check for power down
;	mov	temp, old_pwrcntrl
;	in	old_pwrcntrl, PIND
;	sbrc	old_pwrcntrl, POWER_CONTROL
;	rjmp	NoPathStore
;	eor	temp, old_pwrcntrl
;	andi	temp, (1 << POWER_CONTROL)
;	breq	NoPathStore
;	rcall	PathStore
;NoPathStore:
; check reset path button
	mov	temp, old_resetbtn
	in	old_resetbtn, PIND
	sbrc	old_resetbtn, RESETPATH
	rjmp	NoResetPath
	eor	temp, old_resetbtn
	andi	temp, (1 << RESETPATH)
	breq	NoResetPath
	rcall	ClearPath
NoResetPath:
	out	SREG, st_store2
	reti
;****************
; dt: tmp4,tmp5
TimeToSpeed:
	cpi	tmp4, low(SPEED_CNSTNT / 255)
	mov	temp, tmp5
	sbci	temp, high(SPEED_CNSTNT / 255)
	ldi	tmp1, 255
	brcs	SpeedTooMatch
	ldi	tmp1, SPEED_CNSTNT - SPEED_CNSTNT/256/256*256*256 - (SPEED_CNSTNT - SPEED_CNSTNT/256/256*256*256)/256*256
	ldi	tmp2, (SPEED_CNSTNT - SPEED_CNSTNT/256/256*256*256)/256
	ldi	tmp3, SPEED_CNSTNT/256/256
; div SPEED_CNSTNT/dt
div24u:
	clr	YL	;clear remainder Low byte
	sub	YH, YH;clear remainder High byte and carry
	ldi	temp, 3*8+1	;init loop counter
d24u_1:
	rol	tmp1		;shift left dividend
	rol	tmp2
	rol	tmp3
	dec	temp		;decrement counter
	breq	SpeedResult
d24u_2:
	rol	YL	;shift dividend into remainder
	rol	YH
	sub	YL, tmp4	;remainder = remainder - divisor
	sbc	YH, tmp5
	brcc	d24u_3		;if result negative
	add	YL, tmp4	;    restore remainder
	adc	YH, tmp5
	clc			;    clear carry to be shifted into result
	rjmp	d24u_1		;else
d24u_3:
	sec			;    set carry to be shifted into result
	rjmp	d24u_1
SpeedResult:
SpeedTooMatch:
	ret
;****************
; speed: tmp1
SpeedToView:
; / 100 (0x64)
;	mov	tmp1, speed
	ser	temp
Div100loop:
	subi	tmp1, 0x64
	inc	temp
	brcc	Div100loop
	subi	temp, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_SPEED, temp
	ldi	temp, 0x64
	add	tmp1, temp
;
	ser	temp
Div10loop:
	subi	tmp1, 0x0a
	inc	temp
	brcc	Div10loop
	subi	temp, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_SPEED + 1, temp
	ldi	temp, 0x0a
	add	tmp1, temp
;
	subi	tmp1, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_SPEED + 2, tmp1
	ret
;****************
; pulse to path in km
; path = pulse count * ppp / 100000(/2^5/3125)(/2/50000)
; path_1...path_5 : res1...res6
PathToView:
	cli
	mov	tmp1, path_1
	mov	tmp2, path_2
	mov	tmp3, path_3
	mov	tmp4, path_4
	mov	tmp5, path_5
; compare with maximum (17 48 76 e8 00)
	cpi	tmp1, 0
	ldi	temp, 0xe8
	cpc	tmp2, temp
	ldi	temp, 0x76
	cpc	tmp3, temp
	ldi	temp, 0x48
	cpc	tmp4, temp
	ldi	temp, 0x17
	cpc	tmp5, temp
	brcs	PV_LCD
	subi	tmp1, 0
	sbci	tmp2, 0xe8
	sbci	tmp3, 0x76
	sbci	tmp4, 0x48
	sbci	tmp5, 0x14
	mov	path_1, tmp1
	mov	path_2, tmp2
	mov	path_3, tmp3
	mov	path_4, tmp4
	mov	path_5, tmp5
PV_LCD:
	sei
; / 100 000 / 100 000	(/ 10 000 000 000) (02 54 0b e4 00)
	ser	temp
Div1loop:
;	subi	tmp1, 0x00
	subi	tmp2, 0xe4
	sbci	tmp3, 0x0b
	sbci	tmp4, 0x54
	sbci	tmp5, 0x02
	inc	temp
	brcc	Div1loop
	subi	temp, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_PATH, temp
	ldi	temp, 0xe4
	add	tmp2, temp
	ldi	temp, 0x0b
	adc	tmp3, temp
	ldi	temp, 0x54
	adc	tmp4, temp
	ldi	temp, 0x02
	adc	tmp5, temp
; / 100 000 / 10 000	(/ 1 000 000 000) (3b 9a ca 00)
	ser	temp
Div2loop:
;	subi	tmp1, 0x00
	subi	tmp2, 0xca
	sbci	tmp3, 0x9a
	sbci	tmp4, 0x3b
	sbc		tmp5, zero
	inc	temp
	brcc	Div2loop
	subi	temp, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_PATH + 1, temp
	ldi	temp, 0xca
	add	tmp2, temp
	ldi	temp, 0x9a
	adc	tmp3, temp
	ldi	temp, 0x3b
	adc	tmp4, temp
	adc	tmp5, zero
; / 100 000 / 1 000		(/ 100 000 000) (05 f5 e1 00)
	ser	temp
Div3loop:
;	sub	tmp1, 0x00
	subi	tmp2, 0xe1
	sbci	tmp3, 0xf5
	sbci	tmp4, 0x05
	sbc		tmp5, zero
	inc	temp
	brcc	Div3loop
	subi	temp, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_PATH + 2, temp
	ldi	temp, 0xe1
	add	tmp2, temp
	ldi	temp, 0xf5
	adc	tmp3, temp
	ldi	temp, 0x05
	adc	tmp4, temp
	adc	tmp5, zero
; / 100 000 / 100		(/ 10 000 000) (98 96 80)
	ser	temp
Div4loop:
	subi	tmp1, 0x80
	sbci	tmp2, 0x96
	sbci	tmp3, 0x98
	sbc		tmp4, zero
	inc	temp
	brcc	Div4loop
	subi	temp, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_PATH + 3, temp
	ldi	temp, 0x80
	add	tmp1, temp
	ldi	temp, 0x96
	adc	tmp2, temp
	ldi	temp, 0x98
	adc	tmp3, temp
	adc	tmp4, zero
; / 100 000 / 10		(/ 1 000 000) (0f 42 40)
	ser	temp
Div5loop:
	subi	tmp1, 0x40
	sbci	tmp2, 0x42
	sbci	tmp3, 0x0f
	inc	temp
	brcc	Div5loop
	subi	temp, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_PATH + 4, temp
	ldi	temp, 0x40
	add	tmp1, temp
	ldi	temp, 0x42
	adc	tmp2, temp
	ldi	temp, 0x0f
	adc	tmp3, temp
; / 100 000 / 1		(/ 100 000) (01 86 a0)
	ser	temp
Div6loop:
	subi	tmp1, 0xa0
	sbci	tmp2, 0x86
	sbci	tmp3, 0x01
	inc	temp
	brcc	Div6loop
	subi	temp, -'0'
	sts	LCDbuffSTR1 + LCD_SHIFT_PATH + 5, temp
	ret
;****************
; copy memory off screen buffer to lcd
MemToLCD:
;    		
	ldi	tmp1, 0x80	;    DDRAM=0
	rcall	CommandToLCD
	ldi	ZL, low(LCDbuff)
	ldi	ZH, high(LCDbuff)
	ldi	tmp2, LCD_STR_LENGTH
MemToLCD_str1:
	ld	tmp1, Z+
	rcall	DataToLCD
	dec	tmp2
	brne	MemToLCD_str1
;    
	ldi	tmp1, 0xc0	;    DDRAM=40h
	rcall	CommandToLCD
	ldi	tmp2, LCD_STR_LENGTH
MemToLCD_str2:
	ld	tmp1, Z+
	rcall	DataToLCD
	dec	tmp2
	brne	MemToLCD_str2
	ret
;****************
; temp - address, temp - data
EEPROM_read:
; Wait for completion of previous write
	sbic	EECR, EEPE
	rjmp	EEPROM_read
; Set up address (temp) in address register
	out		EEARL, temp
; Start eeprom read by writing EERE
	sbi		EECR, EERE
; Read data from data register
	in		temp, EEDR
	ret
;******************
; write to eeprom
; temp - address, eeprom_data - data
EEPROM_write:
; Wait for completion of previous write
	sbic	EECR, EEPE
	rjmp	EEPROM_write
	out		EEARL, temp
	out		EEDR, eeprom_data
	cbi		EECR, EEPM0
	cbi		EECR, EEPM1
	in		eeprom_data, SREG
	cli
	sbi		EECR, EEMPE
	sbi		EECR, EEPE
	out		SREG, eeprom_data
	ret
;****************
; LCD init
LCDinit:
;-     
	cbi	PORTA, RS
	cbi	PORTA, E
; 15 
	ldi	tmp1, 150
	rcall	Wait_N
;		
	ldi	tmp1, 0x30
	rcall	CommandToLCD
;		
	rcall	Wait_N50

	ldi	tmp1, 0x30
	rcall	CommandToLCD

	ldi	tmp1, 0x30
	rcall	CommandToLCD

	ldi	tmp1, 0x38;0x3f		; 8  2  5x11, 0x38  5x8
	rcall	CommandToLCD
	ldi	tmp1, 0x08		;    
	rcall	CommandToLCD
	ldi	tmp1, 0x01		;     0   0,    
	rcall	CommandToLCD
;		
	rcall	Wait_N50

	ldi	tmp1, 0x06		;     
	rcall	CommandToLCD
	ldi	tmp1, 0x3f		
	rcall	CommandToLCD
	ldi	tmp1, 0x08
	rcall	CommandToLCD
	ldi	tmp1, 0x0c		;    
	rcall	CommandToLCD
	ldi	tmp1, 0x06
	rcall	CommandToLCD
; copy template to lcd
CopyTemplate:
	ldi	zl, low(2 * Screen)
	ldi	zh, high(2 * Screen)

	ldi	yl, low(LCDbuff)
	ldi	yh, high(LCDbuff)
	ldi	temp, LCD_STR_LENGTH * 2
CopyTemplate_loop:
	lpm	tmp1, z+
	st	y+, tmp1
	dec	temp
	brne	CopyTemplate_loop
	ret
;****************
;  
WriteCharacter:
	ldi	zl, low(2 * Character)
	ldi	zh, high(2 * Character)
	ldi	xh, 4; character quantity
	ldi	xl, 0x40; cgram start address
WriteCharacter_1:
	ldi	yl, 8; symbol height
WC_loop:
	mov	tmp1, xl
	rcall	CommandToLCD
	lpm	tmp1, Z+
	rcall	DataToLCD
	inc	xl
	dec	yl
	brne	WC_loop
	dec	xh
	brne	WriteCharacter_1
	ret	
;****************
;-  N*100.  tmp1, temp
Wait_N50:
	ldi	tmp1, 50
Wait_N:
	rcall	Wait_100mks
	dec	tmp1
	brne	Wait_N
	ret
;****************
;-     ,  -  tmp1,  temp
CommandToLCD:
	cbi	PORTA, RS
	ldi	temp, (3 * QUARTZ / 1000 - 9) / 3
	rcall	Wait_Nmks

	out	PORTB, tmp1
	ldi	temp, (3 * QUARTZ / 1000 - 9) / 3
	rcall	Wait_Nmks

	sbi	PORTA, E
	ldi	temp, (4 * QUARTZ / 1000 - 9) / 3
	rcall	Wait_Nmks
	cbi	PORTA, E
	rjmp	Wait_100mks
;****************
;-     ,  -  tmp1,  temp
DataToLCD:
	sbi	PORTA, RS
	ldi	temp, (3 * QUARTZ / 1000 - 9) / 3
	rcall	Wait_Nmks

	out	PORTB, tmp1
	ldi	temp, (3 * QUARTZ / 1000 - 9) / 3
	rcall	Wait_Nmks
	sbi	PORTA, E
	ldi	temp, (4 * QUARTZ / 1000 - 9) / 3
	rcall	Wait_Nmks
	cbi	PORTA, E
;----------------
; wait 100 mks
Wait_100mks:
	ldi	temp, (100 * QUARTZ / 1000 - 9) / 3
;----------------
; wait N mks, temp - input data, 3...193.5mks@4MHz
; Nreg = (Nmks * QUARTZ / 1000 - 9) / 3
Wait_Nmks:
	dec	temp
	brne	WAIT_Nmks
	ret
;****************
; screen template
Screen:
.db	"000Kmh  000000Km"
.db	"                "
;----
; user's characters
Character:
;!
.db	31,27,27,27,27,31,27,31
;N
.db	31,13, 5, 5, 9, 9,13,31
;0
.db	27,10,21, 4,21,10,27,31
;"
.db	31,31,31,31,31,31,31,31
